@@ -459,7 +459,7 @@ int plugin_initialize(
 
     if (maxsgid > NGROUPS)
     {
-        lcmaps_log(LOG_ERR,"%s: Error: The prefered set maximum of %d Secondary Gid's exceeds the system maximum of NGROUPS witch is set to %d on this system\n", logstr, maxsgid, NGROUPS);
+        lcmaps_log(LOG_ERR,"%s: Error: The preferred set maximum of %d Secondary Gid's exceeds the system maximum of NGROUPS witch is set to %d on this system\n", logstr, maxsgid, NGROUPS);
         goto fail_posix;
     }
     else if (maxsgid == MAX_UNDEFINED)
